<?php
/**
 * Table Definition for tags
 *
 * PHP version 7
 *
 * Copyright (C) Villanova University 2010.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  Db_Table
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Site
 */
namespace fid_bbi\Db\Table;

use VuFind\Db\Table\Tags as BaseTags;
use Zend\Db\Sql\Expression;

/**
 * Table Definition for tags
 *
 * @category VuFind
 * @package  Db_Table
 * @author   Dorian Merz <merz@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org Main Site
 */
class Tags extends BaseTags
{
    /**
     * Get a list of all tags generated by the user in favorites lists.  Note that
     * the returned list WILL NOT include tags attached to records that are not
     * saved in favorites lists.
     *
     * @param string $userId     User ID to look up.
     * @param string $resourceId Filter for tags tied to a specific resource (null
     * for no filter).
     * @param int    $listId     Filter for tags tied to a specific list (null for no
     * filter).
     * @param string $source     Filter for tags tied to a specific record source
     * (null for no filter).
     *
     * @return \Zend\Db\ResultSet\AbstractResultSet
     */
    public function getForUser($userId, $resourceId = null, $listId = null,
        $source = null
    ) {
        $callback = function ($select) use ($userId, $resourceId, $listId, $source) {
            $select->columns(
                [
                    'id' => new Expression(
                        'min(?)', ['tags.id'],
                        [Expression::TYPE_IDENTIFIER]
                    ),
                    'tag' => $this->caseSensitive
                        ? 'tag' : new Expression('lower(tag)'),
                    'cnt' => new Expression(
                        'COUNT(DISTINCT(?))', ['rt.resource_id'],
                        [Expression::TYPE_IDENTIFIER]
                    )
                ]
            );
            $select->join(
                ['rt' => 'resource_tags'], 'tags.id = rt.tag_id', []
            );
            $select->join(
                ['r' => 'resource'], 'rt.resource_id = r.id', []
            );
            $select->join(
                ['ur' => 'user_resource'], 'r.id = ur.resource_id', []
            );
            $select->group(['tag'])->order([new Expression('lower(tag)')]);

            $select->where->equalTo('ur.user_id', $userId)
                ->equalTo('rt.user_id', $userId);

            if (null !== $source) {
                $select->where->equalTo('r.source', $source);
            }

            if (null !== $resourceId) {
                $select->where->equalTo('r.record_id', $resourceId);
            }
            if (null !== $listId) {
                $select->where->equalTo('rt.list_id', $listId);
            }
        };
        return $this->select($callback);
    }
}
