<?php
/**
 * finc specific model for MARC records with a fullrecord in Solr.
 *
 * PHP version 5
 *
 * Copyright (C) Leipzig University Library 2015.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  RecordDrivers
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     http://vufind.org/wiki/vufind2:record_drivers Wiki
 */
namespace fid_bbi\RecordDriver;

use VuFindSearch\Query\Query as Query;

/**
 * finc specific model for MARC records with a fullrecord in Solr.
 *
 * @category VuFind
 * @package  RecordDrivers
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     http://vufind.org/wiki/vufind2:record_drivers Wiki
 */
trait SolrMarcFincTrait
{
    /**
     * Get an array of title detail lines with original notations combining
     * information from MARC field 245 and linked content in 880.
     *
     * @return array
     */
    public function getTitleDetails()
    {
        $title = '';

        if ($field = $this->getMarcRecord()->getField('245')) {
            if ($subfield = $field->getSubfield('a')) {
                // modified due to #13670
                // > Titel: 245$a $n $p $h $b $c
                $title = $subfield->getData();
                foreach (['n', 'p', 'h', 'b'] as $subkey) {
                    if ($subfield = $field->getSubfield($subkey)) {
                        $title .= ' ' . $subfield->getData();
                    }
                }
            }
        }
        if ($field = $this->getMarcRecord()->getField('249')) {
            // 249$a and 249$v are repeatable
            if ($subfields = $field->getSubfields('a')) {
                $vs = $field->getSubfields('v');
                foreach ($subfields as $i => $a) {
                    $title .= '. ' . $a->getData();
                    if (isset($vs[$i])) {
                        $title .= ' / ' . $vs[$i]->getData();
                    }
                }
            }
            // 249$b is non repeatable and applies to all $a$v combinations
            if ($field->getSubfield('b')) {
                $title .= ' : ' . $field->getSubfield('b')->getData();
            }
            // 249$c is non repeatable and applies to all $a$v combinations
            if ($field->getSubfield('c')) {
                $title .= ' / ' . $field->getSubfield('c')->getData();
            }
        }

        return array_merge(
            [$title],
            $this->getLinkedFieldArray('245', ['a', 'b', 'c'])
        );
    }

    public function isSeriesVolume()
    {
        $field = $this->getMarcRecord()->getField('490');

        if ($field && $field->getSubfield('v')) {
            return true;
        }
        return false;
    }

    public function getSeries()
    {
        $field = $this->getMarcRecord()->getField('490');
        if ($field && $this->isSeriesVolume() && $field->getSubfields('a')) {
            $retval = [];
            $tmp = [];
            foreach ($field->getSubfields('a') as $k => $v) {
                $tmp[] = $v->getData();
            }
            $retval['title'] = $tmp;
            $retval['volume'] = $field->getSubfield('v')->getData();
            return $retval;
        }
        return null;
    }

    public function getMarcContainerTitle()
    {
        foreach ($this->getFormats() as $format) {
            if ($result = $this->tryMethod('get' . $format . 'ContainerTitle'))
            {
                return $result;
            }
        }
        return null;
    }

    protected function getSerialComponentPartContainerTitle()
    {
        // refs #16588
        // Zeitschriftenartikel (Medientyp Artikel / format SerialComponentPart):
        // Bsp. https://katalog.fid-bbi.de/Record/183-1678179175#details
        // "Erscheint in:" bitte mit Marc 773 $t", "773 $g befüllen
        // erwartetes Ergebnis: Bibliotheksdienst, 53(2019), 10-11, Seite 667-675
        $containers = [];
        foreach ($this->getMarcRecord()->getFields('773') as $marc773)
        {
            $t = $marc773->getSubfield('t');
            if (empty($t)) continue;
            $text = $t->getData();
            foreach ($marc773->getSubfields('g') as $sub)
            {
                $text .= ', '.$sub->getData();
            }
            $containers[] = $text;
        }
        return $containers;
    }

    protected function getBookComponentPartContainerTitle()
    {
        // Artikel in Büchern (Medientyp Buchkapitel / format BookComponentPart):
        // Bsp. https://katalog.fid-bbi.de/Record/183-166383623X#details
        // "Erscheint in." bitte mit Marc 773$t", "773 $d", S. "936$h befüllen
        // erwartetes Ergebnis: GENial!, Dresden : Media Vista KG, 2017, S. 24-25
        $containers = [];
        foreach ($this->getMarcRecord()->getFields('773') as $marc773)
        {
            $t = $marc773->getSubfield('t');
            if (empty($t)) continue;
            $text = $t->getData();
            $d = $marc773->getSubfield('d');
            if (!empty($d)) $text .= ', '.$d->getData();
            if ($marc936 = $this->getMarcRecord()->getField('936'))
            {
                if ($h = $marc936->getSubfield('h'))
                {
                    $text .= ', S. '.$h->getData();
                }
            }

            $containers[] = $text;
        }
        return $containers;
    }
}
